///-----------------------------------------------------------------------
/// <copyright file="KBDTools.h" company="Honeywell International Inc.">
///     Copyright (c) 2009-2015 Honeywell International Inc. All rights reserved.
/// </copyright>
/// <updated> 
///     Mar 2, 2015
/// </updated>
/// <summary>
///     Header file for Keyboard Remapper Tools
/// </summary>
///-----------------------------------------------------------------------

#pragma once

#ifdef KBDTools_EXPORTS
#define KBDTools_API __declspec(dllexport)
#else
#define KBDTools_API __declspec(dllimport)
#endif

#define Normal 0u /// Represents the normal plane when neither Orange or Green are selected.
#define Orange 1u /// Represents the Orange plane, selected by the Orange key.
#define Green  2u /// Represents the Green plane, selected by the Green key.

/// Key Type.  Used to determine the primary function of the key.
#define NormalKey  0x00u /// Key will be a normal key value.
#define PlaneShift 0x01u /// Key will select the key plane, such as Orange or Green.
#define NamedEvent 0x02u /// Key will trigger a predefined Named Event. 
#define MultiKey   0x04u /// Key will trigger a predefined key macro.
#define Modifier   0x08u /// Key will act as a modifier key, such as Shift, Control, Alt, Caps, etc. 
#define AppLaunch  0x10u /// Key will act as a Microsoft Shell Application Launch key.
#define DLLFunc    0x20u /// Key will trigger a predefined function defined in the keyboard driver.
#define Rotate     0x40u /// Key will select one of several predefined key values, such as ABC on a cell phone 2 key. 
#define Unicode    0x80u /// Key will select one of several predefined Unicode key values. 

/// Key Attribute.  Used to modify the operation of the key.
#define NoAttrib 0x00u /// Value parameter will represent a Scan Code value.
#define Extended 0x01u /// Key is preceded by the extended 0xE0 byte.
#define NoRepeat 0x02u /// Key will not auto repeat.
#define Silent   0x04u /// Key press will not produce a key click.
#define VKEY     0x08u /// Value parameter will represent a VKEY value.
#define NOOP     0x10u /// Key will do nothing.  Disables the key.
#define Shifted  0x20u /// Key is preceded by a left shift key before it's value is sent.
#define NoChord  0x40u /// Multikeys will not be chorded when sent. 

/// Key Modifier.  Used to set the "Sticky" value of the key and control the keyboard LEDs.
#define NoModifier    0x00u  /// No Stickyness and no LED.
#define StickyOnce    0x01u  /// Key remains pressed until next key press of any key.   Intended for Shift, Cap, Orange, etc.
#define StickyPersist 0x02u  /// Key remains pressed until next key press of same key.  Intended for Shift, Cap, Orange, etc.
#define StickyLock    0x04u  /// Key pressed twice in a row remains pressed until next key press of same key.  Otherwise like StickyOnce. Intended for Shift, Cap, Orange, etc.
#define LED1          0x10u  /// Key will also light LED 1.
#define LED2          0x20u  /// Key will also light LED 2.
#define LED3          0x40u  /// Key will also light LED 3.

extern "C"
{
    // Sets a particular keys definition.
    KBDTools_API HRESULT SetKey(BYTE plane, BYTE page, BYTE usage, BYTE keyType, BYTE keyAttribute, BYTE keyModifier, BYTE value);					 

    // Sets a particular keys definition on the specified keyboard.
    KBDTools_API HRESULT SetKeyEx(BYTE keyBd, BYTE plane, BYTE page, BYTE usage, BYTE keyType, BYTE keyAttribute, BYTE keyModifier, BYTE value);					 

    // Retrieves the definition of a key.
    KBDTools_API HRESULT GetKey(BYTE plane, BYTE page, BYTE usage, BYTE* keyType, BYTE* keyAttribute, BYTE* keyModifier, BYTE* value);				 

    // Retrieves the definition of a key on the specified keyboard.
    KBDTools_API HRESULT GetKeyEx(BYTE keyBd, BYTE plane, BYTE page, BYTE usage, BYTE* keyType, BYTE* keyAttribute, BYTE* keyModifier, BYTE* value);				 

    // Retrieves the default definition of a key.
    KBDTools_API HRESULT GetDefaultKey(BYTE plane, BYTE page, BYTE usage, BYTE* keyType, BYTE* keyAttribute, BYTE* keyModifier, BYTE* value);				 

    // Retrieves the default definition of a key on the specified keyboard.
    KBDTools_API HRESULT GetDefaultKeyEx(BYTE keyBd, BYTE plane, BYTE page, BYTE usage, BYTE* keyType, BYTE* keyAttribute, BYTE* keyModifier, BYTE* value);				 

    // Disables a key for a particular plane (Normal, Orange, Green)
    KBDTools_API HRESULT DisableKey(BYTE plane,BYTE page,BYTE usage); 

    // Disables a key for a particular plane (Normal, Orange, Green) on the specified keyboard
    KBDTools_API HRESULT DisableKeyEx(BYTE keyBd, BYTE plane,BYTE page,BYTE usage); 

    // Restores a single key to it's default definition for the indicated plane.
    KBDTools_API HRESULT ResetKey(BYTE plane,BYTE page,BYTE usage);   

    // Restores a single key to it's default definition for the indicated plane on the specified keyboard.
    KBDTools_API HRESULT ResetKeyEx(BYTE keyBd, BYTE plane,BYTE page,BYTE usage);   

    // Restores all key definitions to their default state.
    KBDTools_API HRESULT ResetAll();									 

    // Creates XML file with the definitions of the embedded keyboard.
    KBDTools_API HRESULT ExportKeys(TCHAR* fileName, TCHAR* str, size_t * strLen);		 

    // Creates XML file with the definitions of the on the specified keyboard.
    KBDTools_API HRESULT ExportKeysEx(BYTE keyBd, TCHAR* fileName, TCHAR* str, size_t * strLen);		 

    // Creates XML file with the default definitions of the embedded keyboard.
    KBDTools_API HRESULT ExportDefaultKeys(TCHAR* fileName, TCHAR* str, size_t * strLen);	 

    // Creates XML file with the default definitions on the specified keyboard.
    KBDTools_API HRESULT ExportDefaultKeysEx(BYTE keyBd, TCHAR* fileName, TCHAR* str, size_t * strLen);	 

    // Imports key definitions from a XML file or string.
    KBDTools_API HRESULT ImportKeys(TCHAR* str,bool additive);		 

    // Imports key definitions from a XML file or string on the specified keyboard.
    KBDTools_API HRESULT ImportKeysEx(BYTE keyBd, TCHAR* str,bool additive);		 

    // Gets the state of the auto notify property.
    KBDTools_API HRESULT GetKeyboardNotify(bool* enabled);			 

    // Enables or disables the auto notification of the keyboard driver of key definition changes.
    KBDTools_API HRESULT SetKeyboardNotify(bool enable);				

    // Retrieves the unique ID for the embedded keyboard.
    KBDTools_API HRESULT GetKeyboardID(TCHAR* str, DWORD *len);		 

    // Retrieves the unique ID for the specified keyboard.
    KBDTools_API HRESULT GetKeyboardIDEx(BYTE keyBd, TCHAR* str, DWORD *len);		 

    // Retrieves the version number of the assembly.
    KBDTools_API HRESULT GetVersion(WORD* major, WORD* minor, WORD* build, WORD* revision);

    // Indicates if a key's definition exists
    KBDTools_API HRESULT KeyExists(BYTE plane, BYTE page, BYTE usage, bool* result);  

    // Indicates if a key's definition exists on the specified keyboard
    KBDTools_API HRESULT KeyExistsEx(BYTE keyBd, BYTE plane, BYTE page, BYTE usage, bool* result);  

    // Indicates if a key's default definition exists
    KBDTools_API HRESULT KeyDefaultExists(BYTE plane, BYTE page, BYTE usage, bool* result);   

    // Indicates if a key's default definition exists on the specified keyboard
    KBDTools_API HRESULT KeyDefaultExistsEx(BYTE keyBd, BYTE plane, BYTE page, BYTE usage, bool* result);   
} // extern "C"
