﻿/*
THIS SOURCE CODE AND ANY DOCUMENTATION RELATING TO IT IS PROVIDED "AS IS" WITH
 NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
 PURPOSE. IN NO EVENT WILL HONEYWELL BE LIABLE TO YOU OR ANY OTHER PARTY FOR ANY
 DIRECT, INDIRECT, GENERAL, SPECIAL, INCIDENTAL, CONSEQUENTIAL, EXEMPLARY OR OTHER
 DAMAGES OF ANY KIND ARISING FROM THE USE OF OR INABILITY TO USE THE SOURCE CODE
 OR DERIVATIVE WORK, EVEN IF HONEYWELL HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 DAMAGES. IN NO EVENT SHALL HONEYWELL'S TOTAL LIABILITY EXCEED THE AMOUNT OF THE
 LICENSE FEE PAID FOR THE SOURCE CODE.

	This source code is licensed to you subject to the following conditions:
a.  the license is non-exclusive, non-transferable and  revocable;

b. you may use the source code for the limited purpose of developing applications
 solely for use on Honeywell hardware;

c. you may use the source code or any object programs or executable programs derived
 therefrom only on Honeywell hardware;

d. you may not assign, transfer or sublicense these limited license rights to
 anyone.  Any attempt to do so shall be void;

e. Honeywell may revoke this license immediately in its sole discretion if you fail
 to comply with any of these conditions;

f. you hereby acknowledge that Honeywell is the sole owner of the source code.  All
 proprietary rights in the source code 	(including, without limitation, any and all
 copyright, patent, trade secret, trademark or other proprietary rights) shall
 remain with Honeywell. Title in and to the source code shall remain solely with Honeywell.

g.  you agree that the source code is furnished to you on a strictly confidential,
 limited use basis;

h.  this is a single user license and the source code may only be installed on one computer;

i.  you do not have any right to receive any updates or upgrades to the source code;

j.  you must not obscure or remove any copyright notices, trademarks, labels, legends
 or other intellectual property rights notices appearing on or in the source code. 

Copyright (c) 2003-2015 Honeywell International Inc. All Rights Reserved.

*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Honeywell.Mobility.Print;

namespace SampleXP
{
    public partial class Form1 : Form
    {
        int columns; // Will hold the number of columns a printer can print in normal mode (rounded down to nearest 10)
        string[,] PrinterList;

        bool bFinalPage; // Will hold printing status

        /// <summary>
        /// Constructor for the dialog.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called when form is loaded.
        /// Good place for initialization code.
        /// </summary>
        private void Form1_Load(object sender, EventArgs e)
        {
            // make the window full screen
            this.MaximizeBox = false;
            this.MinimizeBox = false;

            // Load the combo box up with the available printers.
            PrinterList = LinePrinter.EnumeratePrinters();
            for (int i = 0; i < PrinterList.GetLength(0); i++)
            {
                cbPrtrMenu.Items.Add(PrinterList[i, 1]);
            }
            cbPrtrMenu.SelectedIndex = 0;

            StatusMessageBox.Text = "* * * Status Messages * * *\r\n";
        }

        #region Set up printing
        /// <summary>
        /// Starts the print job.
        /// </summary>
        private void btnPrint_Click(object sender, EventArgs e)
        {
            #region Lock the controls during printing
            // Keeps controls from changing during printing
            // Ensures we do not attempt 2 or more simultaneous prints
            btnPrint.Enabled = false;
            btnClear.Enabled = false;
            cbPrintHeaders.Enabled = false;
            cbPrintFooters.Enabled = false;
            #endregion

            #region Determine printer width from printer identifier
            // We do this here by stripping off the number in front of "COL" in the printer identifier.  
            // You can call the printers anything, but I found it handy to put the columns in the identifier.
            string printid = PrinterList[cbPrtrMenu.SelectedIndex, 0];
            int idx = printid.ToUpper().LastIndexOf("COL", StringComparison.CurrentCultureIgnoreCase);
            string col = printid.Substring(idx - 2, 2);
            columns = Convert.ToInt32(col);
            #endregion


            #region Print Report
            // We put the report printing into a loop so that in case of error, we can ask if they want to retry
            // the report printing.
            bool repeat = false;       // Used in case of an exception, tells us if we want to retry the report.
            // Used in the footer routine, in the 80 column report to let us know if we finished the report.
            bFinalPage = false;
            do
            {
                repeat = false;
                bool isException = false;
                try
                {
                    // Create new instance of the LinePrinter
                    lp = new LinePrinter(PrinterList[cbPrtrMenu.SelectedIndex, 0]);

                    #region Add event handlers to the LinePrinter
                    // add a handler for header messages 
                    if (cbPrintHeaders.Checked) lp.HeaderEventEx += new LinePrinter.HeaderEventHandler(PrintHeader);
                    // add a handler for footer messages
                    if (cbPrintFooters.Checked) lp.FooterEventEx += new LinePrinter.FooterEventHandler(PrintFooter);
                    // add a handler for status messages
                    lp.StatusMessageEventEx += new LinePrinter.StatusMessageEventHandler(DisplayStatus);
                    // add a handler for warning messages
                    lp.WarningMessageEventEx += new LinePrinter.WarningMessageEventHandler(WarningHandler);
                    #endregion


                    if ((tbAddress.Text.Length > 0) && ((tbPinPort.Text.Length > 0) && tbPinPort.Text.CompareTo("PIN/PORT") != 0))
                    {
                        lp.Open(tbAddress.Text, int.Parse(tbPinPort.Text)); //Connect to printer
                    }
                    else if (tbAddress.Text.Length > 0)
                    {
                        lp.Open(tbAddress.Text); //Connect to printer
                    }
                    else
                    {
                        lp.Open(); //Connect to printer
                    }

                    btnAbort.Enabled = true; //Enable the abort button

                    #region Print the appropriatly sized report
                    if (columns <= 40)
                    {
                        Print40ColReport();
                    }
                    else if (columns <= 57)
                    {
                        Print57ColReport();
                    }
                    else // 80 Column Report
                    {
                        Print80ColReport();
                    }
                    bFinalPage = true; // For Page Printers (682x, PK80...) let the footer know that we have 
                                       // completed the report.
                    lp.FormFeed(); //Formfeed makes sure we print footers on continous paper at end of report
                    #endregion

                    btnAbort.Enabled = false; //Report completly sent to printer so disable the abort button

                    //Close connection to printer in finally clause

                }
                catch (Exception regex)
                {
                    btnAbort.Enabled = false; //Disable the abort button before showing error.
                    isException = true;
                    repeat = ShowError(regex);
                }
                finally
                {
                    // Make sure we only perform operations on lp if it is not null.  The "new" might have failed.
                    if (lp != null)
                    {
                        // Do Close before removing StatusMessageEventEX if no exception so that we show status from Close.
                        if (!isException) lp.Close();

                        #region Remove event handlers from the LinePrinter
                        // Remove Header Event Handler
                        if (cbPrintHeaders.Checked) lp.HeaderEventEx -= new LinePrinter.HeaderEventHandler(PrintHeader);
                        // Remove Footer Event Handler
                        if (cbPrintFooters.Checked) lp.FooterEventEx -= new LinePrinter.FooterEventHandler(PrintFooter);
                        // Remove Warning Message Handler
                        lp.WarningMessageEventEx -= new LinePrinter.WarningMessageEventHandler(WarningHandler);
                        // remove handler for status messages
                        lp.StatusMessageEventEx -= new LinePrinter.StatusMessageEventHandler(DisplayStatus);
                        #endregion

                        // Do Close after removing StatusMessageEventEX if exception so that we do not show status updates from
                        // close.  This is purely a cosmetic decision.
                        if (isException) lp.Close();
                    }
                }
            } while (repeat);
            #endregion

            #region Unlock the controls that were locked earlier
            cbPrintHeaders.Enabled = true;
            cbPrintFooters.Enabled = true;
            btnPrint.Enabled = true;
            btnClear.Enabled = true;
            #endregion
        }
        #endregion

        #region 40Col 60Col and 80Col Report Print Routines
        // Dummy sample reports

        /// <summary>
        /// Prints a report formatted to 40 columns for printers capable of printing between 40 and 59 columns.
        /// </summary>
        private void Print40ColReport() // PB20, PB2, etc
        {
            lp.Write(" PRD. DESCRIPT.   PRC.  QTY.    NET.");
            lp.NewLine();
            lp.NewLine();

            lp.Write(" 1501 Timer-Md1  13.15     1   13.15");
            lp.NewLine();
            lp.Write(" 1502 Timer-Md2  13.15     3   39.45");
            lp.NewLine();
            lp.Write(" 1503 Timer-Md3  13.15     1   13.15");
            lp.NewLine();
            lp.Write(" 1504 Timer-Md4  13.15     4   52.60");
            lp.NewLine();
            lp.Write(" 1505 Timer-Md5  13.15     5   65.75");
            lp.NewLine();
            lp.Write("                        ----  ------");
            lp.NewLine();
            lp.Write("              SUBTOTAL    15  197.25");
            lp.NewLine();
            lp.NewLine();

            lp.Write("                        ----  ------");
            lp.NewLine();
            lp.Write("           TOTAL SALES    15  197.15");
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();

            lp.Write("               PRODUCT        179.25");
            lp.NewLine();
            lp.Write("               DEPOSIT         18.00");
            lp.NewLine();
            lp.Write("                              ------");
            lp.NewLine();
            lp.Write("              SUBTOTAL        197.25");
            lp.NewLine();
            lp.Write("      SUBTOTAL CREDITS          0.00");
            lp.NewLine();
            lp.Write("           OTHER ITEMS          2.14");
            lp.NewLine();
            lp.Write("                 LABOR          4.44");
            lp.NewLine();
            lp.Write("          5% State Tax          9.86");
            lp.NewLine();
            lp.NewLine();

            lp.Write("                              ------");
            lp.NewLine();
            lp.Write("           BALANCE DUE        213.74");
            lp.NewLine();
            lp.NewLine();

            lp.Write(" PAYMENT TYPE: CASH");
            lp.NewLine();
            lp.NewLine();

            lp.Write("       SIGNATURE / STORE STAMP");
            lp.NewLine();
            lp.NewLine();

            lp.BoldOn();
            lp.Write("              ORIGINAL");
            lp.BoldOff();
            lp.NewLine();
        }

        /// <summary>
        /// Prints a report formatted to 60 columns for printers capable of printing between 60 and 79 columns.
        /// </summary>
        private void Print57ColReport()  // PB3, PB31, PB32
        {
            lp.Write("Part#       DESCRIPTION   QTY   PRICE DEPOSIT         NET");
            lp.NewLine();
            lp.NewLine();

            lp.BoldOn();
            lp.Write("                            SALES");
            lp.NewLine();
            lp.BoldOff();
            lp.NewLine();

            lp.Write("1501     Timer Module 1     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1502     Timer Module 2     3   11.95    1.20       39.45");
            lp.NewLine();
            lp.Write("1503     Timer Module 3     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1504     Timer Module 4     4   11.95    1.20       52.60");
            lp.NewLine();
            lp.Write("1505     Timer Module 5     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1506     Timer Module 6     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1507     Timer Module 7     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1508     Timer Module 8     1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1509     Timer Module 9     1   11.95    1.20       13.15");
            lp.NewLine();

            lp.Write("                          ____                    _______");
            lp.NewLine();
            lp.Write("    SUBTOTAL               15                      197.25");
            lp.NewLine();
            lp.NewLine();
            lp.Write("1501     Plug-in Module 1   1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1502     Plug-in Module 2   3   11.95    1.20       39.45");
            lp.NewLine();
            lp.Write("1503     Plug-in Module 3   1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1504     Plug-in Module 4   4   11.95    1.20       52.60");
            lp.NewLine();
            lp.Write("1505     Plug-in Module 5   2   11.95    1.20       26.30");
            lp.NewLine();
            lp.Write("1506     Plug-in Module 6   1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1507     Plug-in Module 7   1   11.95    1.20       13.15");
            lp.NewLine();
            lp.Write("1508     Plug-in Module 8   1   11.95    1.20       13.15");
            lp.NewLine();

            lp.Write("                         ____                     _______");
            lp.NewLine();
            lp.Write("   SUBTOTAL                15                      197.25");
            lp.NewLine();
            lp.Write("                         ____                     _______");
            lp.NewLine();
            lp.Write("   TOTAL SALES             30                      394.50");
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();

            // make sure the next 15 lines will not be split across a page.
            lp.NoOrphan(15, LinePrinter.OrphanMode.OM_CURRENT);


            lp.Write("                   PRODUCT                         394.50");
            lp.NewLine();
            lp.Write("                   DEPOSIT                          36.00");
            lp.NewLine();
            lp.Write("                                                  _______");
            lp.NewLine();
            lp.Write("                   SUBTOTAL                        394.50");
            lp.NewLine();
            lp.Write("                   SUBTOTAL CREDITS                  0.00");
            lp.NewLine();
            lp.Write("                   OTHER ITEMS                       2.19");
            lp.NewLine();
            lp.Write("                   LABOR                             4.44");
            lp.NewLine();
            lp.Write("                   5% State Tax                     19.72");
            lp.NewLine();
            lp.Write("                                                  _______");
            lp.NewLine();

            lp.Write("                  BALANCE DUE: CHARGE              420.81");
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();

            lp.Write("                SIGNATURE / STORE STAMP");
            lp.NewLine();
            lp.Write("");
            lp.NewLine();
            lp.NewLine();
        }

        /// <summary>
        /// Prints a report formatted to 80 columns for printers capable of printing at least 80 columns.
        /// </summary>
        private void Print80ColReport()  // PB4x, 6820, PB50, etc
        {
            lp.Write("  Part#               DESCRIPTION         QTY    PRICE  DEPOSIT         NET");
            lp.NewLine();
            lp.NewLine();

            lp.BoldOn();
            lp.Write("                                         SALES");
            lp.NewLine();
            lp.BoldOff();
            lp.NewLine();

            lp.Write("   1501               Timer Module 1        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1502               Timer Module 2        3    11.95    1.20          39.45");
            lp.NewLine();
            lp.Write("   1503               Timer Module 3        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1504               Timer Module 4        4    11.95    1.20          52.60");
            lp.NewLine();
            lp.Write("   1505               Timer Module 5        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1506               Timer Module 6        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1507               Timer Module 7        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1508               Timer Module 8        1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1509               Timer Module 9        1    11.95    1.20          13.15");
            lp.NewLine();

            lp.Write("                                         ____                    _____________");
            lp.NewLine();
            lp.Write("             SUBTOTAL                      15                          197.25");
            lp.NewLine();
            lp.Write("   1501               Plug-in Module 1      1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1502               Plug-in Module 2      3    11.95    1.20          39.45");
            lp.NewLine();
            lp.Write("   1503               Plug-in Module 3      1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1504               Plug-in Module 4      4    11.95    1.20          52.60");
            lp.NewLine();
            lp.Write("   1505               Plug-in Module 5      2    11.95    1.20          26.30");
            lp.NewLine();
            lp.Write("   1506               Plug-in Module 6      1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1507               Plug-in Module 7      1    11.95    1.20          13.15");
            lp.NewLine();
            lp.Write("   1508               Plug-in Module 8      1    11.95    1.20          13.15");
            lp.NewLine();

            lp.Write("                                         ____                    _____________");
            lp.NewLine();
            lp.Write("             SUBTOTAL                      15                          197.25");
            lp.NewLine();
            lp.Write("                                         ____                    _____________");
            lp.NewLine();
            lp.Write("             TOTAL SALES                   30                          394.50");
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();

            // make sure the next 15 lines will not be split across a page.
            lp.NoOrphan(15, LinePrinter.OrphanMode.OM_CURRENT);


            lp.Write("                                    PRODUCT                            394.50");
            lp.NewLine();
            lp.Write("                                    DEPOSIT                             36.00");
            lp.NewLine();
            lp.Write("                                                                     ________");
            lp.NewLine();
            lp.Write("                                    SUBTOTAL                           394.50");
            lp.NewLine();
            lp.Write("                                    SUBTOTAL CREDITS                     0.00");
            lp.NewLine();
            lp.Write("                                    OTHER ITEMS                          2.19");
            lp.NewLine();
            lp.Write("                                    LABOR                                4.44");
            lp.NewLine();
            lp.Write("                                    5% State Tax                        19.72");
            lp.NewLine();
            lp.Write("                                                                    _________");
            lp.NewLine();

            lp.Write("                                    BALANCE DUE: CHARGE                 420.81");
            lp.NewLine();
            lp.NewLine();
            lp.NewLine();

            lp.Write("                               SIGNATURE / STORE STAMP");
            lp.NewLine();
            lp.Write("");
            lp.NewLine();
            lp.NewLine();


            lp.Write("                                   Thank You For");
            lp.NewLine();
            lp.Write("                                   Your Business");
            lp.NewLine();
            lp.Write("                            Comments or Questions, Please");
            lp.NewLine();
            lp.Write("                             Call (555) 555-1234 M-F 7-5");
            lp.NewLine();
        }
        #endregion

        #region Header and Footer Functions
        /// <summary>
        /// Prints the beginning of page header for the report.
        /// </summary>
        public void PrintHeader(object source, EventArgs evnt)
        {
            if (columns <= 40)
            {
                lp.Write(DateTime.Now.ToString());
                lp.NewLine();
                lp.Write("    ROUTE# 123456  SALESREP# 54321");
                lp.NewLine();
                lp.NewLine();

                lp.BoldOn();
                lp.Write("             SALES ORDER");
                lp.BoldOff();
                lp.NewLine();
                lp.NewLine();

                lp.Write("CUSTOMER: Casual Step");
                lp.NewLine();
                lp.Write("          Distributing Co.");
                lp.NewLine();
                lp.Write("          550 2nd Street S.E.");
                lp.NewLine();
                lp.Write("          Cedar Rapids, IA 52401-1234");
                lp.NewLine();
                lp.NewLine();

                lp.Write("DOCUMENT# 123456789012");
                lp.NewLine();
                lp.Write("  TAX ID# AB4001");
                lp.NewLine();
                lp.NewLine();

                lp.Write("START TIME: 13:15");
                lp.NewLine();
                lp.Write("  END TIME: 13:30");
                lp.NewLine();
                lp.NewLine();

            }
            else if (columns <= 57)
            {
                lp.Write("LOCATION# 123456 REP# 123456 " + DateTime.Now.ToString() + " PAGE " + lp.PageNumber.ToString());
                lp.NewLine();

                lp.BoldOn();
                lp.Write("                      Casual Step");
                lp.NewLine();
                lp.Write("                    Distributing Co.");
                lp.NewLine();
                lp.BoldOff();
                lp.Write("                  550 2nd Street S.E.");
                lp.NewLine();
                lp.Write("              Cedar Rapids, IA 52401-1234");
                lp.NewLine();
                lp.NewLine();

                lp.BoldOn();
                lp.Write("CUSTOMER#   123456   Grant Adams   ");
                lp.Write("DOCUMENT# 123456789012");
                lp.NewLine();
                lp.BoldOff();

                lp.Write("START TIME: 09:45    Box 123              TAX ID# AB4001");
                lp.NewLine();
                lp.Write("END TIME: 09:46    Tampa, FL");
                lp.NewLine();
                lp.NewLine();
                lp.NewLine();
            }
            else // columns > 57
            {
                lp.Write("LOCATION# 123456 SERVICE # 1233456 SERVICE REP# 123456 " + DateTime.Now.ToString() + " PAGE ");
                lp.Write(lp.PageNumber.ToString());
                lp.NewLine();

                lp.BoldOn();
                lp.Write("                                  Casual Step");
                lp.NewLine();
                lp.Write("                                Distributing Co.");
                lp.NewLine();
                lp.BoldOff();
                lp.Write("                               550 2nd Street S.E.");
                lp.NewLine();
                lp.Write("                             Cedar Rapids, IA 52401-1234");
                lp.NewLine();
                lp.NewLine();

                lp.Write("CUSTOMER#   123456   Grant Adams                        ");
                lp.BoldOn();
                lp.Write("DOCUMENT# 123456789012");
                lp.NewLine();
                lp.BoldOff();

                lp.Write("START TIME: 09:45    Box 123                            TAX ID# AB4001");
                lp.NewLine();
                lp.Write("END TIME:   09:46    Tampa, FL");
                lp.NewLine();
                lp.NewLine();
                lp.NewLine();
            }
        }

        /// <summary>
        /// Prints the end of page footer for the report.
        /// </summary>
        public void PrintFooter(object source, EventArgs evnt)
        {
            if (columns <= 40)
            {
                lp.Write("            Thank You For");
                lp.NewLine();
                lp.Write("            Your Business");
                lp.NewLine();
                lp.Write("     Comments or Questions, Please");
                lp.NewLine();
                lp.Write("      Call (555) 555-1234 M-F 7-5");
                lp.NewLine();
            }
            else if (columns <= 57)
            {
                lp.Write("                    Thank You For");
                lp.NewLine();
                lp.Write("                    Your Business");
                lp.NewLine();
                lp.Write("             Comments or Questions, Please");
                lp.NewLine();
                lp.Write("              Call (555) 555-1234 M-F 7-5");
                lp.NewLine();
            }
            else //columns > 57
            // For our 80 column sample, we assume that we have pages of set length and want to print "CONTINUED" on
            // all but the last page.
            {
                lp.BoldOn();

                if (bFinalPage)
                    lp.Write("                                     ORIGINAL");
                else
                    lp.Write("                                     CONTINUED");

                lp.BoldOff();
                lp.NewLine();
            }
        }
        #endregion

        /// <summary>
        /// Handles any NPCP errors that occur.
        /// </summary>
        public void WarningHandler(object source, LinePrinter.WarningMessageArgs Error)
        {
            string temp;
            DialogResult result;

            temp = string.Format("{0} from {1} Printer \n\r\n\r Fix problem and continue printing?", NPCPErrorStr(Error.ErrorCode), Error.DeviceType);
            result = MessageBox.Show(temp, "PRINTER WARNING", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);

            Application.DoEvents();  // makes the MessageBox go away faster.

            if (result == DialogResult.No)
            {
                // May want to consider a try / catch here, but only exception will be from
                // invoking the status event.
                lp.Cancel();
            }
        }

        /// <summary>
        /// Adds the received status to the status box.
        /// </summary>
        public void DisplayStatus(object source, LinePrinter.StatusMessageArgs Status)
        {
            string temp;
            switch (Status.MessageType)
            {
                case LinePrinter.StatusMessageTypes.SM_NONE: temp = "NONE"; break;
                case LinePrinter.StatusMessageTypes.SM_WRITEINCOMPLETE: temp = "WRITEINCOMPLETE"; break;
                case LinePrinter.StatusMessageTypes.SM_STARTDOC: temp = "STARTDOC"; break;
                case LinePrinter.StatusMessageTypes.SM_STARTPAGE: temp = "STARTPAGE"; break;
                case LinePrinter.StatusMessageTypes.SM_ENDPAGE: temp = "ENDPAGE"; break;
                case LinePrinter.StatusMessageTypes.SM_ENDDOC: temp = "ENDDOC"; break;
                case LinePrinter.StatusMessageTypes.SM_USERCANCEL: temp = "USERCANCEL"; break;
                case LinePrinter.StatusMessageTypes.SM_CANCEL: temp = "CANCEL"; break;
                case LinePrinter.StatusMessageTypes.SM_FINISHED: temp = "FINISHED"; break;
                case LinePrinter.StatusMessageTypes.SM_COMPLETE: temp = "COMPLETE"; break;
                default: temp = "UNKNOWN"; break;
            }
            StatusMessageBox.Text += "Status " + temp + "\r\n";
        }

        /// <summary>
        /// Translates the NPCP error code into a string to be displayed.
        /// </summary>
        static string NPCPErrorStr(int NPCPErr)
        {
            switch (NPCPErr)
            {
                case 102: return "Printer Not Ready";
                case 104: return "Not Receiving";
                case 106: return "Not Transmitting";
                case 113: return "MAC Length Parity Error";
                case 120: return "Invalid Frame Received";
                case 121: return "NS != VR";
                case 122: return "NR != VS";
                case 123: return "Receive Length Error";
                case 124: return "CRC Error";
                case 200: return "Frame Reject";
                case 201: return "Invalid Frame Rejected";
                case 202: return "NR Mismatch";
                case 203: return "NS Mismatch";
                case 204: return "Disconnect";
                case 210: return "Bind Error";
                case 221: return "Invalid PLDU";
                case 222: return "Head Jam";
                case 223: return "Paper Out";
                case 224: return "Low Voltage";
                case 225: return "Over Voltage";
                case 226: return "Low Battery";
                case 227: return "Cover Off";
                case 228: return "Print Head Error";
                case 229: return "Paper Feed Error";
                default: return "Unkown";
            }
        }

        /// <summary>
        /// Displays any printer errors.
        /// </summary>
        public bool ShowError(Exception e)
        {
            bool retry = true;
            string Message = "UNKNOWN ERROR WHILE PRINTING\n\r\n\rRETRY REPORT?";

            if (e is LinePrinterException)
            {
                LinePrinterException lpe = (LinePrinterException)e;
                if (lpe.ErrorType == LinePrinterException.ErrorTypes.CONFIG_ERROR)
                {
                    Message = "PRINTER CONFIGURATION ERROR\r\n\r\nPossibly invalid configuration file!";
                    retry = false; // set to false, so we will not ask if they want to continue below.
                }
                else if (lpe.ErrorType == LinePrinterException.ErrorTypes.PRINTER_ERROR)
                {
                    if (lpe.PortName == "NPCP") Message = string.Format("NPCP PRINTER ERROR\n\r\n\rError # {0} - {1}\n\r\n\rRETRY REPORT?", lpe.ErrorCode, NPCPErrorStr(lpe.ErrorCode));
                    if (lpe.PortName == "IRDA") Message = "IRDA PRINTER ERROR\n\r\n\rCheck Printer Alignment\n\r\n\rRETRY REPORT?";
                    if (lpe.PortName.StartsWith("COM")) Message = "SERIAL PRINTER ERROR\n\r\n\rCheck Printer Connection\n\r\n\rRETRY REPORT?";
                    if (lpe.PortName == "WPPORT") Message = "WPPORT PRINTER ERROR\n\r\n\rCheck Printer\n\r\n\rRETRY REPORT?";
                    if (lpe.PortName == "WIFI") Message = "WI-FI PRINTER ERROR\n\r\n\rCheck Printer\n\r\n\rRETRY REPORT?";
                }
            }
            if (retry)  // If false, we already decided that we are not going to retry.
            {
                // Ask to see if we want to retry the report.
                if (MessageBox.Show(Message, "REPORT ERROR!", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.No) retry = false;
            }
            else MessageBox.Show(Message, "FATAL REPORT ERROR!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

            return retry;
        }

        /// <summary>
        /// Initiates an abort of the print job.
        /// </summary>
        private void bAbort_Click(object sender, EventArgs e)
        {
            if (lp != null) lp.Cancel();

        }

        /// <summary>
        /// Clears the status screen.
        /// </summary>
        private void bClear_Click(object sender, EventArgs e)
        {
            StatusMessageBox.Text = "* * * Status Messages * * *\r\n";
        }

        private void tbAddress_Enter(object sender, EventArgs e)
        {
            if (tbAddress.Text.CompareTo("Address") == 0)
            {
                tbAddress.Text = "";
                tbAddress.ForeColor = Color.Black;
            }
        }

        private void tbAddress_Leave(object sender, EventArgs e)
        {
            if (tbAddress.Text.CompareTo("") == 0)
            {
                tbAddress.Text = "Address";
                tbAddress.ForeColor = Color.DarkGray;
            }
        }

        private void tbPinPort_Enter(object sender, EventArgs e)
        {
            if (tbPinPort.Text.CompareTo("PIN/PORT") == 0)
            {
                tbPinPort.Text = "";
                tbPinPort.ForeColor = Color.Black;
            }
        }

        private void tbPinPort_Leave(object sender, EventArgs e)
        {
            if (tbAddress.Text.CompareTo("") == 0)
            {
                tbPinPort.Text = "PIN/PORT";
                tbPinPort.ForeColor = Color.DarkGray;
            }
        }
    }
}
